
if (typeof Polyfillr === 'undefined') {
	Polyfillr = {};
}

Console = Polyfillr.Console = (function(global) {

	var _console = global.console;
	var _TAB_STR = ' ' + new Array(128).join(' ');
	var _DIV_STR = '=' + new Array(128).join('=');



	/*
	 * HELPERS
	 */

	var _Table = function(labels, values) {

		this.labels = labels;
		this.values = values;

	};

	var _render = function(args) {

		var mode = 0;
		if (!(args instanceof Array)) {
			args = [ args ];
			mode = 1;
		}


		var chars = [];

		for (var a = 0, al = args.length; a < al; a++) {

			var data = args[a];
			if (typeof data === 'boolean') {
				chars.push('\u001b[94m' + data);
			} else if (typeof data === 'number') {
				chars.push('\u001b[95m' + data);
			} else if (typeof data === 'string') {
				if (mode === 1) {
					chars.push('\u001b[92m"' + _render_string(data) + '"');
				} else {
					chars.push('\u001b[39m' + _render_string(data) + '');
				}
			} else if (typeof data === 'null' || data === null) {
				chars.push('\u001b[95mnull');
			} else if (typeof data === 'undefined' || data === undefined) {
				chars.push('\u001b[95mundefined');
			} else if (data instanceof Error) {
				chars.push('\u001b[33m' + _render_error(data));
			} else if (typeof data === 'object') {

				if (data instanceof _Table) {
					chars.push(_render_table(data));
				} else if (data instanceof Array) {
					chars.push(_render_array(data));
				} else if (data instanceof Object) {
					chars.push(_render_object(data));
				}

			} else if (typeof data === 'function') {

				chars.push(_render_function(data));

			}

		}


		return chars.join('');

	};

	var _render_array = function(array, t) {

		var tab   = typeof t === 'number' ? t : 0;
		var lines = [];

		lines.push(_TAB_STR.substr(0, tab) + '[');

		tab += 4;

		for (var a = 0, al = array.length; a < al; a++) {

			var data = array[a];
			var str  = '';

			if (typeof data === 'boolean') {
				str += '\u001b[94m' + data;
			} else if (typeof data === 'number') {
				str += '\u001b[95m' + data;
			} else if (typeof data === 'string') {
				str += '\u001b[92m"' + _render_string(data) + '"';
			} else if (typeof data === 'null' || data === null) {
				str += '\u001b[95mnull';
			} else if (typeof data === 'undefined' || data === undefined) {
				str += '\u001b[95mundefined';
			} else if (typeof data === 'object') {

				if (data instanceof Array) {
					str += _render_array(data,  tab).trim();
				} else if (data instanceof Object) {
					str += _render_object(data, tab).trim();
				}

			} else if (typeof data === 'function') {

				var tmp = (data).toString();
				var i1  = tmp.indexOf('(');
				var i2  = tmp.indexOf(')');
				if (i1 !== -1 && i2 !== -1) {
					str += '\u001b[95mfunction(\u001b[93m' + tmp.substr(i1 + 1, i2 - i1 - 1) + '\u001b[39m)';
				} else {
					str += '\u001b[95mfunction()';
				}

			}

			lines.push(_TAB_STR.substr(0, tab) + str + '\u001b[39m,');

		}

		if (lines.length > 1) {

			var last = lines[lines.length - 1];
			if (last.length > 0) {
				lines[lines.length - 1] = last.substr(0, last.length - 1);
			}

		}

		tab -= 4;

		lines.push(_TAB_STR.substr(0, tab) + ']');

		return lines.join('\n');

	};

	var _render_error = function(err) {

		var lines = [];

		if (typeof err.stack === 'string') {

			var data = err.stack.split('\n');
			var type = data[0].split(':')[0];
			var msg  = data[0].split(':').slice(1).join(':');

			lines.push('\u001b[93m' + type + '\u001b[39m: "' + msg.trim() + '"');


			var stack = data.slice(1);
			if (stack.length > 0) {

				for (var s = 0, sl = stack.length; s < sl; s++) {

					var tmp = stack[s].trim().split(' ');
					if (tmp[0] === 'at') {

						var href = tmp[1].split(':');
						var name = tmp[1].split('/').pop().split(':');
						var line = null;

						if (/[0-9]+/g.test(href[href.length - 1])) href.pop();
						if (/[0-9]+/g.test(href[href.length - 1])) href.pop();
						if (/[0-9]+/g.test(name[name.length - 1])) line = name.pop();
						if (/[0-9]+/g.test(name[name.length - 1])) line = name.pop();


						if (line !== null) {
							name = name.join(':') + '#L' + line;
						} else {
							name = name.join(':');
						}

						if (tmp[1].indexOf('://') !== -1) {
							lines.push('        at ' + name);
						} else {
							lines.push('        at ' + tmp[1]);
						}

					} else {
						lines.push('        ' + tmp.join(' '));
					}

				}

			}

		} else {

			// TODO: Simple Error

		}


		return lines.join('\n');

	};

	var _render_function = function(data) {

		return data;

	};

	var _render_object = function(object, t) {

		var tab   = typeof t === 'number' ? t : 0;
		var lines = [];

		lines.push(_TAB_STR.substr(0, tab) + '{');

		tab += 4;

		for (var prop in object) {

			if (object.hasOwnProperty(prop)) {

				var data = object[prop];
				var str  = '';

				if (typeof data === 'boolean') {
					str += '\u001b[94m' + data;
				} else if (typeof data === 'number') {
					str += '\u001b[95m' + data;
				} else if (typeof data === 'string') {
					str += '\u001b[92m"' + _render_string(data) + '"';
				} else if (typeof data === 'null' || data === null) {
					str += '\u001b[95mnull';
				} else if (typeof data === 'undefined' || data === undefined) {
					str += '\u001b[95mundefined';
				} else if (typeof data === 'object') {

					if (data instanceof Array) {
						str += _render_array(data,  tab).trim();
					} else if (data instanceof Object) {
						str += _render_object(data, tab).trim();
					}

				} else if (typeof data === 'function') {

					var tmp = (data).toString();
					var i1  = tmp.indexOf('(');
					var i2  = tmp.indexOf(')');
					if (i1 !== -1 && i2 !== -1) {
						str += '\u001b[95mfunction(\u001b[93m' + tmp.substr(i1 + 1, i2 - i1 - 1) + '\u001b[39m)';
					} else {
						str += '\u001b[95mfunction()';
					}

				}


				lines.push(_TAB_STR.substr(0, tab) + '\u001b[39m' + prop + ': ' + str + '\u001b[39m,');

			}

		}

		if (lines.length > 1) {

			var last = lines[lines.length - 1];
			if (last.length > 0) {
				lines[lines.length - 1] = last.substr(0, last.length - 1);
			}

		}

		tab -= 4;

		lines.push(_TAB_STR.substr(0, tab) + '}');

		return lines.join('\n');

	};

	var _render_string = function(str) {

		str = str.split('\t').join('\\t');
		str = str.split('\n').join('\\n');

		return str;

	};


	var _render_table = function(table) {

		var cell_w = 8;
		var lines  = [];
		var sizes  = [];
		var labels = table.labels;
		var values = table.values;


		if (values instanceof Array) {

			values.forEach(function(value, v) {

				labels.forEach(function(label) {

					var tmp1 = ' ' + _render(value[label]) + '\u001b[39m ';
					var tmp2 = tmp1.split('\u001b').map(function(val) {
						return val.substr(0, 1) === '[' ? val.substr(4) : val;
					}).join('');

					cell_w = Math.max(tmp2.length, label.length, cell_w);

				});

			});


			var header = ' (index) | ' + labels.map(function(label) {
				return label + _TAB_STR.substr(0, cell_w - label.length - 2);
			}).join(' | ');

			lines.push(header);
			lines.push(_DIV_STR.substr(0, header.length));


			var chunks = values.map(function(value, v) {

				return '\u001b[39m' + labels.map(function(label) {

					var tmp1 = ' ' + _render(value[label]) + '\u001b[39m ';
					var tmp2 = tmp1.split('\u001b').map(function(val) {
						return val.substr(0, 1) === '[' ? val.substr(4) : val;
					}).join('');

					var space = _TAB_STR.substr(0, cell_w - tmp2.length);

					return tmp1 + space;

				}).join('|');

			});

			values.forEach(function(value, v) {

				var i     = ' ' + v;
				var space = _TAB_STR.substr(0, 9 - i.length);

				lines.push(i + space + '|' + chunks[v]);

			});

		} else if (values instanceof Object) {

			for (var v in values) {

				if (values.hasOwnProperty(v)) {

					var value = values[v];

					labels.forEach(function(label) {

						var tmp1 = ' ' + _render(value[label]) + '\u001b[39m ';
						var tmp2 = tmp1.split('\u001b').map(function(val) {
							return val.substr(0, 1) === '[' ? val.substr(4) : val;
						}).join('');

						cell_w = Math.max(tmp2.length, label.length, cell_w);

					});

				}

			}


			var header = ' (index) | ' + labels.map(function(label) {
				return label + _TAB_STR.substr(0, cell_w - label.length - 2);
			}).join(' | ');

			lines.push(header);
			lines.push(_DIV_STR.substr(0, header.length));


			for (var v in values) {

				if (values.hasOwnProperty(v)) {

					var value = values[v];
					var chunk = '\u001b[39m' + labels.map(function(label) {

						var tmp1 = ' ' + _render(value[label]) + '\u001b[39m ';
						var tmp2 = tmp1.split('\u001b').map(function(val) {
							return val.substr(0, 1) === '[' ? val.substr(4) : val;
						}).join('');

						var space = _TAB_STR.substr(0, cell_w - tmp2.length);

						return tmp1 + space;

					}).join('|');


					var i     = ' ' + v;
					var space = _TAB_STR.substr(0, 9 - i.length);

					lines.push(i + space + '|' + chunk);

				}

			}

		}

		return '\n' + lines.join('\n') + '\n';

	};

	var _write_stdout = function(args) {

		var str = _render(args);
		if (str.length > 0) {
			process.stdout.write(str + '\n\u001b[0m');
		}

	};

	var _write_stderr = function(args) {

		var str = _render(args);
		if (str.length > 0) {
			process.stderr.write(str + '\n');
		}

	};



	/*
	 * IMPLEMENTATION
	 */

	var Console = function(id) {

		id = typeof id === 'string' ? id : 'default';


		this.__id     = id;
		this.__timers = {};

	};


	Console.prototype = {

		assert: function(condition) {

			if (!condition) {

				var al   = arguments.length;
				var args = [];

				for (var a = 1; a < al; a++) {
					args.push(arguments[a]);
				}

				this.log.apply(this, args);

			}

		},

		clear: function() {

			process.stdout.write('\x1B[2J\x1B[0f');

		},

		error: function() {

			var al   = arguments.length;
			var args = [ ' (E) ' ];
			for (var a = 0; a < al; a++) {
				args.push(arguments[a]);
			}

			args.reverse();
			args.push('\u001b[41m');
			args.reverse();
			args.push('\u001b[49m');

			_write_stderr(args);

		},

		info: function() {

			var al   = arguments.length;
			var args = [ ' (I) ' ];
			for (var a = 0; a < al; a++) {
				args.push(arguments[a]);
			}

			args.reverse();
			args.push('\u001b[42m');
			args.reverse();
			args.push('\u001b[49m');

			_write_stdout(args);

		},

		log: function() {

			var al   = arguments.length;
			var args = [ ' (L) ' ];
			for (var a = 0; a < al; a++) {
				args.push(arguments[a]);
			}

			_write_stdout(args);

		},

		table: function(data, labels) {

			data   = typeof data === 'object' ? data   : null;
			labels = labels instanceof Array  ? labels : null;


			if (data !== null) {

				if (labels === null) {

					var check = {};
					for (var prop in data) {

						if (data.hasOwnProperty(prop)) {
							check = data[prop];
							break;
						}

					}

					labels = Object.keys(check).sort(function(a, b) {

						var a1 = a.toLowerCase();
						var b1 = b.toLowerCase();

						if (a1 < b1) return -1;
						if (a1 > b1) return  1;

						return 0;

					});

				}


				_write_stdout([ new _Table(labels, data) ]);

			}

		},

		time: function(label) {

			label = typeof label === 'string' ? label : 'default';


			this.__timers[label] = Date.now();

		},

		timeEnd: function(label) {

			label = typeof label === 'string' ? label : 'default';


			var timer = this.__timers[label] || null;
			if (timer !== null) {

				var diff = (Date.now() - timer).toPrecision(3);
				var args = [ ' (T) ', label + ': ' + diff + 'ms' ];

				args.reverse();
				args.push('\u001b[44m');
				args.reverse();
				args.push('\u001b[49m');

				_write_stdout(args);

			}

		},

		warn: function() {

			var al   = arguments.length;
			var args = [ ' (W) ' ];
			for (var a = 0; a < al; a++) {
				args.push(arguments[a]);
			}

			args.reverse();
			args.push('\u001b[43m');
			args.reverse();
			args.push('\u001b[49m');

			_write_stdout(args);

		}

	};


	// global.console = new Console('default');


	return Console;

})(typeof global !== 'undefined' ? global : this);

