if (typeof Array.from !== 'function') {

	Array.from = function(alike/*, predicate, thisArg */) {

		if (alike === null || alike === undefined) {
			throw new TypeError('Array.from requires an array-like object - not null or undefined');
		}


		var construct = this;
		var list      = Object(alike);
		var predicate = arguments.length > 1 ? arguments[1] : void 0;
		var thisArg   = arguments.length > 2 ? arguments[2] : void 0;

		if (typeof predicate !== 'undefined') {

			if (typeof predicate !== 'function') {
				throw new TypeError('Array.from: when provided, the second argument must be a function');
			}

		}

		var length = list.length >>> 0;
		var array  = typeof construct === 'function' ? Object(new construct(length)) : new Array(length);

		for (var i = 0; i < length; i++) {

			var value = list[i];

			if (predicate !== undefined) {

				if (thisArg === undefined) {
					array[i] = predicate(value, i);
				} else {
					array[i] = predicate.call(thisArg, value, i);
				}

			} else {
				array[i] = value;
			}

		}

		array.length = length;

		return array;

	};

}
if (typeof Array.prototype.concat !== 'function') {

	Array.prototype.concat = function(/* [value1 [, ...[, valueN]]]*/) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.concat called on null or undefined');
		}


		var list   = Object(this);
		var length = list.length >>> 0;
		var result = [];
		var value;


		for (var i = 0; i < length; i++) {

			if (i in list) {
				result.push(list[i]);
			}

		}

		for (var a = 0, al = arguments.length; a < al; a++) {

			value = arguments[a];

			if (value instanceof Array) {

				for (var v = 0, vl = value.length; v < vl; v++) {
					result.push(value[v]);
				}

			} else {

				result.push(arguments[a]);

			}

		}


		return result;

	};

}
if (typeof Array.prototype.copyWithin !== 'function') {

	Array.prototype.copyWithin = function(target, start/*, end*/) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.copyWithin called on null or undefined');
		}


		var list   = Object(this);
		var length = list.length >>> 0;

		var tmp1 = target >> 0;
		var tmp2 = start  >> 0;
		var tmp3 = arguments[2] === undefined ? length : arguments[2] >> 0;


		var to   = tmp1 < 0 ? Math.max(length + tmp1, 0) : Math.min(tmp1, length);
		var from = tmp2 < 0 ? Math.max(length + tmp2, 0) : Math.min(tmp2, length);
		var last = tmp3 < 0 ? Math.max(length + tmp3, 0) : Math.min(tmp3, length);


		var count     = Math.min(last - from, length - to);
		var direction = 1;

		if (from < to && to < (from + count)) {
			direction  = -1;
			from      += count - 1;
			to        += count - 1;
		}


		while (count > 0) {

			if (from in list) {
				list[to] = list[from];
			} else {
				delete list[to];
			}

			from += direction;
			to   += direction;
			count--;

		}


		return list;

	};

}
if (typeof Array.prototype.every !== 'function') {

	Array.prototype.every = function(predicate/*, thisArg */) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.every called on null or undefined');
		}

		if (typeof predicate !== 'function') {
			throw new TypeError('predicate must be a function');
		}


		var list    = Object(this);
		var length  = list.length >>> 0;
		var thisArg = arguments.length >= 2 ? arguments[1] : void 0;
		var value;


		for (var i = 0; i < length; i++) {

			if (i in list) {

				value = list[i];

				if (!!predicate.call(thisArg, value, i, list) === false) {
					return false;
				}

			}

		}


		return true;

	};

}
if (typeof Array.prototype.fill !== 'function') {

	Array.prototype.fill = function(value/*, start = 0, end = this.length */) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.fill called on null or undefined');
		}


		var list      = Object(this);
		var length    = list.length >>> 0;
		var start     = arguments[1];
		var end       = arguments[2];
		var rel_start = start === undefined ?      0 : start >> 0;
		var rel_end   = end === undefined   ? length : end >> 0;


		var i_start = rel_start < 0 ? Math.max(length + rel_start, 0) : Math.min(rel_start, length);
		var i_end   = rel_end < 0   ? Math.max(length + rel_end, 0)   : Math.min(rel_end, length);

		for (var i = i_start; i < i_end; i++) {
			list[i] = value;
		}


		return list;

	};

}
if (typeof Array.prototype.filter !== 'function') {

	Array.prototype.filter = function(predicate/*, thisArg */) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.filter called on null or undefined');
		}

		if (typeof predicate !== 'function') {
			throw new TypeError('predicate must be a function');
		}


		var list    = Object(this);
		var length  = list.length >>> 0;
		var thisArg = arguments.length >= 2 ? arguments[1] : void 0;
		var result  = [];
		var value;


		for (var i = 0; i < length; i++) {

			if (i in list) {

				value = list[i];

				if (!!predicate.call(thisArg, value, i, list) === true) {
					result.push(value);
				}

			}

		}


		return result;

	};

}
if (typeof Array.prototype.find !== 'function') {

	Array.prototype.find = function(predicate/*, thisArg */) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.find called on null or undefined');
		}

		if (typeof predicate !== 'function') {
			throw new TypeError('predicate must be a function');
		}


		var list    = Object(this);
		var length  = list.length >>> 0;
		var thisArg = arguments.length >= 2 ? arguments[1] : void 0;
		var value;

		for (var i = 0; i < length; i++) {

			if (i in list) {

				value = list[i];

				if (predicate.call(thisArg, value, i, list)) {
					return value;
				}

			}

		}


		return undefined;

	};

}
if (typeof Array.prototype.findIndex !== 'function') {

	Array.prototype.findIndex = function(predicate/*, thisArg */) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.findIndex called on null or undefined');
		}

		if (typeof predicate !== 'function') {
			throw new TypeError('predicate must be a function');
		}


		var list    = Object(this);
		var length  = list.length >>> 0;
		var thisArg = arguments.length >= 2 ? arguments[1] : void 0;
		var value;


		for (var i = 0; i < length; i++) {

			if (i in list) {

				value = list[i];

				if (!!predicate.call(thisArg, value, i, list) === true) {
					return i;
				}

			}

		}


		return -1;

	};

}
if (typeof Array.prototype.forEach !== 'function') {

	Array.prototype.forEach = function(predicate/*, thisArg */) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.forEach called on null or undefined');
		}

		if (typeof predicate !== 'function') {
			throw new TypeError('predicate must be a function');
		}


		var list    = Object(this);
		var length  = list.length >>> 0;
		var thisArg = arguments.length >= 2 ? arguments[1] : void 0;
		var value;


		for (var i = 0; i < length; i++) {

			if (i in list) {

				value = list[i];

				predicate.call(thisArg, value, i, list);

			}

		}


		return undefined;

	};

}
if (typeof Array.prototype.includes !== 'function') {

	Array.prototype.includes = function(search/*, from */) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.includes called on null or undefined');
		}


		var list   = Object(this);
		var length = list.length >>> 0;
		var from   = arguments.length >= 2 ? arguments[1] : 0;
		var value;


		if (length === 0 || from >= length) {
			return false;
		}


		var start = Math.max(from >= 0 ? from : (length - Math.abs(from)), 0);

		for (var i = start; i < length; i++) {

			if (i in list) {

				value = list[i];

				if (value === search || (isNaN(value) && isNaN(search))) {
					return true;
				}

			}

		}


		return false;

	};

}
if (typeof Array.prototype.indexOf !== 'function') {

	Array.prototype.indexOf = function(search/*, from */) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.indexOf called on null or undefined');
		}


		var list   = Object(this);
		var length = list.length >>> 0;
		var from   = arguments.length >= 2 ? (arguments[1] | 0) : 0;


		if (length === 0 || from >= length) {
			return -1;
		}


		var start = Math.max(from >= 0 ? from : (length - Math.abs(from)), 0);

		for (var i = start; i < length; i++) {

			if (i in list) {

				if (list[i] === search) {
					return i;
				}

			}

		}


		return -1;

	};

}
if (typeof Array.prototype.join !== 'function') {

	Array.prototype.join = function() {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.join called on null or undefined');
		}


		var list      = Object(this);
		var length    = list.length >>> 0;
		var separator = arguments.length >= 1 ? (arguments[0]).toString() : ',';
		var value     = '';
		var tmp;


		for (var i = 0; i < length; i++) {

			if (i in list) {

				tmp = list[i];

				if (tmp !== undefined && tmp !== null) {
					value += (list[i]).toString();
				}

			}

			if (i !== length - 1) {
				value += separator;
			}

		}


		return value;

	};

}
if (typeof Array.prototype.lastIndexOf !== 'function') {

	Array.prototype.lastIndexOf = function(search/*, from */) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.lastIndexOf called on null or undefined');
		}


		var list   = Object(this);
		var length = list.length >>> 0;
		var from   = arguments.length >= 2 ? (arguments[1] | 0) : 0;


		if (length === 0 || from >= length) {
			return -1;
		}


		var start = length - 1;
		if (from !== 0) {
			start = (from > 0 ? from : -1) * Math.abs(from);
		}


		for (var i = (start >= 0 ? Math.min(start, length - 1) : (length - Math.abs(start))); i >= 0; i--) {

			if (i in list) {

				if (list[i] === search) {
					return i;
				}

			}

		}


		return -1;

	};

}
if (typeof Array.prototype.map !== 'function') {

	Array.prototype.map = function(predicate/*, thisArg */) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.map called on null or undefined');
		}

		if (typeof predicate !== 'function') {
			throw new TypeError('predicate must be a function');
		}


		var list    = Object(this);
		var length  = list.length >>> 0;
		var thisArg = arguments.length >= 2 ? arguments[1] : void 0;
		var result  = [];
		var value;


		for (var i = 0; i < length; i++) {

			if (i in list) {

				value     = list[i];
				result[i] = predicate.call(thisArg, value, i, list);

			}

		}


		return result;

	};

}
if (typeof Array.prototype.pop !== 'function') {

	Array.prototype.pop = function() {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.pop called on null or undefined');
		}


		var list   = Object(this);
		var length = list.length >>> 0;
		var value  = undefined;

		var i = length - 1;
		if (i in list) {

			value = list[i];
			delete list[i];

			this.length = i;

		}


		return value;

	};

}
if (typeof Array.prototype.push !== 'function') {

	Array.prototype.push = function(/* [element1 [, ...[, elementN]]]*/) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.push called on null or undefined');
		}


		var list   = Object(this);
		var length = list.length >>> 0;


		for (var a = 0, al = arguments.length; a < al; a++) {
			list[length++] = arguments[a];
		}


		return length;

	};

}
if (typeof Array.prototype.reduce !== 'function') {

	Array.prototype.reduce = function(predicate/*, initialValue */) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.reduce called on null or undefined');
		}

		if (typeof predicate !== 'function') {
			throw new TypeError('predicate must be a function');
		}


		var list   = Object(this);
		var length = list.length >>> 0;
		var start  = 0;
		var value  = undefined;


		if (arguments.length >= 2) {

			value = arguments[1];

		} else {

			while (start < length && !(start in list)) {
				start++;
			}

			if (start >= length) {
				throw new TypeError('Reduce of empty array with no initial value');
			}

			value = list[start++];

		}


		for (var i = start; i < length; i++) {

			if (i in list) {
				value = predicate(value, list[i], i, list);
			}

		}


		return value;

	};

}
if (typeof Array.prototype.reduceRight !== 'function') {

	Array.prototype.reduceRight = function(predicate/*, initialValue */) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.reduceRight called on null or undefined');
		}

		if (typeof predicate !== 'function') {
			throw new TypeError('predicate must be a function');
		}


		var list   = Object(this);
		var length = list.length >>> 0;
		var start  = length - 1;
		var value  = undefined;


		if (arguments.length >= 2) {

			value = arguments[1];

		} else {

			while (start >= 0 && !(start in list)) {
				start--;
			}

			if (start < 0) {
				throw new TypeError('Reduce of empty array with no initial value');
			}

			value = list[start--];

		}


		for (var i = start; i >= 0; i--) {

			if (i in list) {
				value = predicate(value, list[i], i, list);
			}

		}


		return value;

	};

}
if (typeof Array.prototype.slice !== 'function') {

	Array.prototype.slice = function(/* start, end */) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.slice called on null or undefined');
		}


		var list   = Object(this);
		var length = list.length >>> 0;
		var start  = arguments.length >= 1 ? arguments[0] : 0;
		var end    = arguments.length >= 2 ? arguments[1] : length;
		var result = [];


		if (end < 0) {
			end = length - Math.abs(end);
		}


		for (var i = start; i < end; i++) {

			if (i in list) {
				result.push(list[i]);
			}

		}


		return result;

	};

}
if (typeof Array.prototype.some !== 'function') {

	Array.prototype.some = function(predicate/*, thisArg */) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.some called on null or undefined');
		}

		if (typeof predicate !== 'function') {
			throw new TypeError('predicate must be a function');
		}


		var list    = Object(this);
		var length  = list.length >>> 0;
		var thisArg = arguments.length >= 2 ? arguments[1] : void 0;


		for (var i = 0; i < length; i++) {

			if (i in list) {

				if (!!predicate.call(thisArg, list[i], i, list) === true) {
					return true;
				}

			}

		}


		return false;

	};

}
if (typeof Array.prototype.splice !== 'function') {

	Array.prototype.splice = function(/* start, count, ... elements */) {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.splice called on null or undefined');
		}


		var list     = Object(this);
		var length   = list.length >>> 0;
		var elements = [];
		var result   = [];


		if (arguments.length === 0) {
			return result;
		}


		var start = 0;
		var count = 0;

		if (arguments.length >= 1) {
			start = (arguments[0] | 0);
		}

		if (arguments.length >= 2) {
			count = (arguments[1] | 0);
		} else {
			count = length - start;
		}


		for (var a = 2; a < arguments.length; a++) {
			elements.push(arguments[a]);
		}


		var diff = count;

		for (var i = 0; i < diff; i++) {

			var k = start + i;
			if (k in list) {

				result.push(list[k]);
				delete list[k];

			}

		}


		for (var e = 0, el = elements.length; e < el; e++) {

			var k = start + e;

			if (k in list) {

				list[k + 1] = list[k];
				list[k] = elements[e];

			} else {

				list[k] = elements[e];

			}

		}


		for (var i = length - 1; i > 0; i--) {

			if (!((i - 1) in list)) {

				if (i in list) {
					list[i - 1] = list[i];
					delete list[i];
				}

			}

		}


		return result;

	};

}
if (typeof Array.prototype.toLocaleString !== 'function') {

	Array.prototype.toLocaleString = function() {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.toLocaleString called on null or undefined');
		}


		var list   = Object(this);
		var length = list.length >>> 0;
		var value  = '';
		var tmp;


		for (var i = 0; i < length; i++) {

			if (i in list) {

				tmp = list[i];

				if (tmp !== undefined && tmp !== null) {

					if (typeof tmp.toLocaleString === 'function') {
						value += tmp.toLocaleString();
					} else {
						value += tmp.toString();
					}

				}

			}

			if (i !== length - 1) {
				value += ',';
			}

		}


		return value;

	};

}
if (typeof Array.prototype.toSource !== 'function') {

	Array.prototype.toSource = function() {

		var list   = Object(this);
		var length = list.length >>> 0;
		var value  = '';
		var entry;


		for (var i = 0; i < length; i++) {

			if (i in list) {

				entry = list[i];

				if (entry === undefined) {
					value += 'undefined';
				} else if (entry === null) {
					value += 'null';
				} else if (typeof entry === 'number') {
					value += (entry).valueOf();
				} else if (typeof entry === 'string') {
					value += '"' + entry + '"';
				} else if (entry instanceof RegExp) {
					value += (entry).toString();
				} else if (typeof (entry).toSource === 'function') {
					value += (entry).toSource();
				} else {
					value += (entry).toString();
				}

			}

			if (i !== length - 1) {
				value += ', ';
			}

		}


		return '[' + value + ']';

	};

}
if (typeof Array.prototype.toString !== 'function') {

	Array.prototype.toString = function() {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.toString called on null or undefined');
		}


		var list   = Object(this);
		var length = list.length >>> 0;
		var value  = '';
		var tmp;


		for (var i = 0; i < length; i++) {

			if (i in list) {

				tmp = list[i];

				if (tmp !== undefined && tmp !== null) {
					value += tmp.toString();
				}

			}

			if (i !== length - 1) {
				value += ',';
			}

		}


		return value;

	};

}
if (typeof Array.prototype.unique !== 'function') {

	Array.prototype.unique = function() {

		if (this === null || this === undefined) {
			throw new TypeError('Array.prototype.unique called on null or undefined');
		}


		var clone  = [];
		var list   = Object(this);
		var length = this.length >>> 0;
		var value;

		for (var i = 0; i < length; i++) {

			value = list[i];

			if (clone.indexOf(value) === -1) {
				clone.push(value);
			}
		}

		return clone;

	};

}
if (typeof Array.toSource !== 'function') {

	Array.toSource = function() {
		return 'function Array() {\n\t[native code]\n}';
	};

}
if (typeof Boolean.prototype.toJSON !== 'function') {

	Boolean.prototype.toJSON = function() {
		return this.valueOf();
	};

}
if (typeof Boolean.prototype.toSource !== 'function') {

	Boolean.prototype.toSource = function() {
		return '(new Boolean(' + (this).valueOf() + '))';
	};

}
if (typeof Boolean.toSource !== 'function') {

	Boolean.toSource = function() {
		return 'function Boolean() {\n\t[native code]\n}';
	};

}
if (typeof Date.prototype.toJSON !== 'function') {

	var _format_date = function(n) {
		return n < 10 ? '0' + n : '' + n;
	};

	Date.prototype.toJSON = function() {

		if (isFinite(this.valueOf()) === true) {

			return this.getUTCFullYear()             + '-' +
				_format_date(this.getUTCMonth() + 1) + '-' +
				_format_date(this.getUTCDate())      + 'T' +
				_format_date(this.getUTCHours())     + ':' +
				_format_date(this.getUTCMinutes())   + ':' +
				_format_date(this.getUTCSeconds())   + 'Z';

		}


		return null;

	};

}
if (typeof JSON.query !== 'function') {

	JSON.query = function(object, query) {

		query = typeof query === 'string' ? query : '';


		var pointer = null;

		if (object instanceof Object) {

			var references = query.split('>');
			var check      = references[0] || '';

			if (check !== '' && references.length > 0) {

				pointer = object;

				for (var r = 0, rl = references.length; r < rl; r++) {

					var ref = references[r].trim();
					if (ref !== '') {

						if (pointer[ref] !== undefined) {

							pointer = pointer[ref];

						} else {

							var num = parseInt(ref, 10);
							if (!isNaN(num) && pointer[num] !== undefined) {

								pointer = pointer[num];

							} else {

								pointer = null;
								break;

							}

						}

					} else {

						pointer = null;
						break;

					}

				}


			} else {

				pointer = object;

			}

		}

		return pointer;

	};

}
if (typeof Number.EPSILON !== 'number') {

	Number.EPSILON = 2.220446049250313e-16;

}
if (typeof Number.MAX_SAFE_INTEGER !== 'number') {

	Number.MAX_SAFE_INTEGER = 9007199254740991;

}
if (typeof Number.MIN_SAFE_INTEGER !== 'number') {

	Number.MIN_SAFE_INTEGER = -9007199254740991;

}
if (typeof Number.isFinite !== 'function') {

	Number.isFinite = function(value) {

		if (typeof value === 'number') {

			if (value === -Infinity || value === +Infinity) {
				return false;
			} else if (value !== value) {
				return false;
			}


			return true;

		}


		return false;

	};

}
if (typeof Number.isInteger !== 'function') {

	Number.isInteger = function(value) {

		if (typeof value === 'number') {

			if (value === -Infinity || value === +Infinity) {
				return false;
			} else if (Math.floor(value) === value) {
				return true;
			}

		}


		return false;

	};

}
if (typeof Number.isNaN !== 'function') {

	Number.isNaN = function(value) {

		if (typeof value === 'number') {
			return value !== value;
		}


		return false;

	};

}
if (typeof Number.isSafeInteger !== 'function') {

	Number.isSafeInteger = function(value) {

		if (typeof value === 'number') {

			if (Math.floor(value) === value) {

				if (value === -Infinity || value === +Infinity) {
					return false;
				} else if (Math.abs(value) <= Number.MAX_SAFE_INTEGER) {
					return true;
				}
			}

		}


		return false;

	};

}
if (typeof Number.prototype.toJSON !== 'function') {

	Number.prototype.toJSON = function() {
		return this.valueOf();
	};

}
if (typeof Number.prototype.toLocaleString !== 'function') {

	Number.prototype.toLocaleString = function() {

		var lang = 'en-US';
		var tmp1 = (this | 0).toString();
		var tmp2 = (this - (this | 0)).toString().substr(2);

		if (typeof navigator === 'object') {

			if (typeof navigator.language === 'string') {
				lang = navigator.language;
			}

		}


		var sep_1 = ',';
		var sep_2 = '.';

		if (/^de|it|es/.test(lang)) {
			sep_1 = '.';
			sep_2 = ',';
		} else if (/^fr/.test(lang)) {
			sep_1 = ' ';
			sep_2 = ',';
		}

		for (var t = 0; t < tmp1.length; t++) {

			if (t % 3 === 0 && tmp1[tmp1.length - t] !== sep_1) {
				tmp1 = tmp1.substr(0, tmp1.length - t) + sep_1 + tmp1.substr(tmp1.length - t + 1);
			}

		}


		return tmp1 + sep_2 + tmp2;

	};

}
if (typeof Number.prototype.toSource !== 'function') {

	Number.prototype.toSource = function() {
		return '(new Number(' + (this).valueOf() + '))';
	};

}
if (typeof Number.toSource !== 'function') {

	Number.toSource = function() {
		return 'function Number() {\n\t[native code]\n}';
	};

}
if (typeof Object.assign !== 'function') {

	Object.assign = function(object /*, ... sources */) {

		if (object !== Object(object)) {
			throw new TypeError('Object.assign called on a non-object');
		}


		for (var a = 1, al = arguments.length; a < al; a++) {

			var source = arguments[a];
			if (source !== undefined && source !== null) {

				for (var key in source) {

					if (Object.prototype.hasOwnProperty.call(source, key) === true) {
						object[key] = source[key];
					}

				}

			}

		}


		return object;

	};

}
if (typeof Object.entries !== 'function') {

	Object.entries = function(object) {

		if (object !== Object(object)) {
			throw new TypeError('Object.entries called on a non-object');
		}


		var values = [];

		for (var prop in object) {

			if (Object.prototype.hasOwnProperty.call(object, prop)) {
				values.push([ prop, object[prop] ]);
			}

		}

		return values;

	};

}
if (typeof Object.filter !== 'function') {

	Object.filter = function(object, predicate/*, thisArg */) {

		if (object !== Object(object)) {
			throw new TypeError('Object.filter called on a non-object');
		}

		if (typeof predicate !== 'function') {
			throw new TypeError('predicate must be a function');
		}


		var props   = [];
		var values  = [];
		var thisArg = arguments.length >= 3 ? arguments[2] : void 0;

		for (var prop in object) {

			var value = object[prop];

			if (Object.prototype.hasOwnProperty.call(object, prop)) {

				if (predicate.call(thisArg, value, prop, object)) {
					props.push(prop);
					values.push(value);
				}

			}

		}


		var filtered = {};

		for (var i = 0; i < props.length; i++) {
			filtered[props[i]] = values[i];
		}

		return filtered;

	};

}
if (typeof Object.find !== 'function') {

	Object.find = function(object, predicate/*, thisArg */) {

		if (object !== Object(object)) {
			throw new TypeError('Object.find called on a non-object');
		}

		if (typeof predicate !== 'function') {
			throw new TypeError('predicate must be a function');
		}


		var thisArg = arguments.length >= 3 ? arguments[2] : void 0;

		for (var prop in object) {

			var value = object[prop];

			if (Object.prototype.hasOwnProperty.call(object, prop)) {

				if (predicate.call(thisArg, value, prop, object)) {
					return value;
				}

			}

		}

		return undefined;

	};

}
if (typeof Object.is !== 'function') {

	Object.is = function(a, b) {

		if (a === 0 && b === 0) {

			return (1 / a) === (1 / b);

		} else if (a === b) {

			return true;

		} else if (Number.isNaN(a) && Number.isNaN(b)) {

			return true;

		}


		return false;

	};

}
if (typeof Object.keys !== 'function') {

	Object.keys = function(object) {

		if (object !== Object(object)) {
			throw new TypeError('Object.keys called on a non-object');
		}


		var keys = [];

		for (var prop in object) {

			if (Object.prototype.hasOwnProperty.call(object, prop)) {
				keys.push(prop);
			}

		}

		return keys;

	};

}
if (typeof Object.map !== 'function') {

	Object.map = function(object, predicate/*, thisArg */) {

		if (object !== Object(object)) {
			throw new TypeError('Object.map called on a non-object');
		}

		if (typeof predicate !== 'function') {
			throw new TypeError('predicate must be a function');
		}


		var clone   = {};
		var keys    = Object.keys(object).sort();
		var length  = keys.length >>> 0;
		var thisArg = arguments.length >= 3 ? arguments[2] : void 0;
		var key;
		var value;
		var tmp;


		for (var k = 0; k < length; k++) {

			key   = keys[k];
			value = object[key];
			tmp   = predicate.call(thisArg, value, key, object);

			if (tmp !== undefined) {
				clone[key] = tmp;
			}

		}


		return clone;

	};

}
if (typeof Object.prototype.toLocaleString !== 'function') {

	// XXX: Yes, this is the correct implementation. WTF.
	Object.prototype.toLocaleString = function() {
		return Object.prototype.toString.call(this);
	};

}
if (typeof Object.prototype.toSource !== 'function' && typeof Object.defineProperty === 'function') {

	var _array_to_source  = Array.prototype.toSource || null;
	var _string_to_source = String.prototype.toSource || null;


	Object.defineProperty(Object.prototype, 'toSource', {

		enumerable: false,
		value:      function() {

			var prototype   = Object.getPrototypeOf(this);
			var konstruktor = this.constructor;


			if (this === Array || this === String || this === RegExp) {

				var name = this.toString().split(' ')[1];

				return 'function ' + name + ' {\n\t[native code]\n}';

			} else if (this === Object && prototype === Object.__proto__) {

				return 'function Object() {\n\t[native code]\n}';

			} else if (konstruktor === Array) {

				if (_array_to_source !== null) {
					return _array_to_source.call(this);
				} else {
					return "" + this;
				}

			} else if (konstruktor === Boolean) {

				return '(new Boolean(' + (this).valueOf() + '))';

			} else if (konstruktor === Number) {

				return '(new Number(' + (this).valueOf() + '))';

			} else if (konstruktor === String) {

				if (_string_to_source !== null) {
					return _string_to_source.call(this);
				} else {
					return "" + this;
				}

			} else if (konstruktor === RegExp) {

				return "" + this;

			} else if (konstruktor === Object && prototype !== null && Object.getPrototypeOf(prototype) === null) {

				var list  = Object(this);
				var last  = Object.keys(list).pop();
				var value = '';
				var entry;


				for (var i in list) {

					if (Object.prototype.hasOwnProperty.call(list, i)) {

						entry = list[i];

						value += i + ':';

						if (entry === undefined) {

							value += 'undefined';

						} else if (entry === null) {

							value += 'null';

						} else if (typeof entry === 'number') {

							value += (entry).valueOf();

						} else if (typeof entry === 'string') {

							value += '"' + entry + '"';

						} else if (typeof entry === 'function') {

							if (typeof entry.toSource === 'function') {
								value += entry.toSource();
							} else {
								value += entry.toString();
							}

						} else if (entry instanceof Array || entry instanceof RegExp || entry instanceof String) {

							if (typeof entry.toSource === 'function') {
								value += entry.toSource();
							} else {
								value += entry.toString();
							}

						} else if (entry instanceof Object) {

							// XXX: Surrounding expression brackets
							var tmp = entry.toSource();
							value += tmp.substr(1, tmp.length - 2);

						} else {

							value += (entry).toString();

						}


						if (i !== last) {
							value += ', ';
						}

					}

				}


				return '({' + value + '})';

			}

		}

	});

}
if (typeof Object.sort !== 'function') {

	Object.sort = function(object) {

		if (object !== Object(object)) {
			throw new TypeError('Object.sort called on a non-object');
		}


		var clone  = {};
		var keys   = Object.keys(object).sort();
		var length = keys.length >>> 0;
		var key;
		var value;

		for (var k = 0; k < length; k++) {

			key   = keys[k];
			value = object[key];

			if (value instanceof Array) {

				clone[key] = value.map(function(element) {

					if (element instanceof Array) {
						return element;
					} else if (element instanceof Object) {
						return Object.sort(element);
					} else {
						return element;
					}

				});

			} else if (value instanceof Object) {

				clone[key] = Object.sort(value);

			} else {

				clone[key] = value;

			}

		}

		return clone;

	};

}
if (typeof Object.toSource !== 'function') {

	Object.toSource = function() {
		return 'function Object() {\n\t[native code]\n}';
	};

}
if (typeof Object.values !== 'function') {

	Object.values = function(object) {

		if (object !== Object(object)) {
			throw new TypeError('Object.values called on a non-object');
		}


		var values = [];

		for (var prop in object) {

			if (Object.prototype.hasOwnProperty.call(object, prop)) {
				values.push(object[prop]);
			}

		}

		return values;

	};

}
if (typeof RegExp.prototype.toSource !== 'function') {

	RegExp.prototype.toSource = function() {

		if (this.constructor === RegExp) {
			return this.toString();
		} else {
			return "function RegExp() {\n\t[native code]\n}";
		}

	};

}
if (typeof RegExp.toSource !== 'function') {

	RegExp.toSource = function() {
		return 'function RegExp() {\n\t[native code]\n}';
	};

}
if (typeof String.fromCodePoint !== 'function') {

	String.fromCodePoint = function(/* ... nums */) {

		var units = [];
		var code;
		var hi;
		var lo;

		var length = arguments.length >>> 0;
		if (length === 0) {
			return '';
		}


		for (var i = 0; i < length; i++) {

			code = Number(arguments[i]);

			if (!isFinite(code) || code < 0 || code > 0x10ffff || ((code | 0) !== code)) {

				throw new RangeError('Invalid code point: ' + code);

			} else {

				if (code <= 0xffff) {

					units.push(code);

				} else {

					code -= 0x10000;
					hi    = (code >> 10)   + 0xD800;
					lo    = (code % 0x400) + 0xDC00;
					units.push(hi, lo);

				}


				if (i + 1 === length || units.length > 0x4000) {
					break;
				}

			}

		}


		return String.fromCharCode.apply(null, units);

	};

}
if (typeof String.prototype.codePointAt !== 'function') {

	String.prototype.codePointAt = function(index) {

		if (this === null || this === undefined) {
			throw new TypeError('String.prototype.codePointAt called on null or undefined');
		}


		var string = String(this);
		var length = string.length >>> 0;
		var i      = typeof index === 'number' ? (index | 0) : 0;

		if (i < 0 || i >= length) {
			return undefined;
		}


		var char_01 = string.charCodeAt(i);
		var char_02;

		if (char_01 >= 0xD800 && char_01 <= 0xDBFF && length > i + 1) {

			char_02 = string.charCodeAt(i + 1);

			if (char_02 >= 0xDC00 && char_02 <= 0xDFFF) {
				return (char_01 - 0xD800) * 0x400 + (char_02 - 0xDC00) + 0x10000;
			}

		}


		return char_01;

	};

}
if (typeof String.prototype.endsWith !== 'function') {

	String.prototype.endsWith = function(search/*, from */) {

		if (this === null || this === undefined) {
			throw new TypeError('String.prototype.endsWith called on null or undefined');
		}


		var value  = (this).toString();
		var from   = arguments.length >= 2 ? (arguments[1] | 0) : value.length;
		var tmp    = String(search);
		var length = tmp.length >>> 0;


		var chunk = value.substr(from - length);
		if (chunk === tmp) {
			return true;
		}


		return false;

	};

}
if (typeof String.prototype.includes !== 'function') {

	String.prototype.includes = function(search/*, from */) {

		if (this === null || this === undefined) {
			throw new TypeError('String.prototype.includes called on null or undefined');
		}


		var value  = String(this);
		var from   = arguments.length >= 2 ? (arguments[1] | 0) : 0;
		var tmp    = String(search);
		var length = tmp.length >>> 0;

		if (from + length > value.length) {
			return false;
		}


		return value.indexOf(search, from) !== -1;

	};

}
if (typeof String.prototype.padEnd !== 'function') {

	String.prototype.padEnd = function(length/*, chunk */) {

		if (this === null || this === undefined) {
			throw new TypeError('String.prototype.padEnd called on null or undefined');
		}


		length = length >> 0;


		var chunk = arguments.length >= 2 ? String(arguments[1] || '') : '';
		var value = (this).toString();

		if (this.length > length) {
			return value;
		}


		var tmp = length - this.length;
		if (tmp > chunk.length) {
			chunk += chunk.repeat(tmp / chunk.length);
		}

		return value + chunk.slice(0, tmp);

	};

}
if (typeof String.prototype.padStart !== 'function') {

	String.prototype.padStart = function(length/*, chunk */) {

		if (this === null || this === undefined) {
			throw new TypeError('String.prototype.padStart called on null or undefined');
		}


		length = length >> 0;


		var chunk = arguments.length >= 2 ? String(arguments[1] || '') : '';
		var value = (this).toString();

		if (this.length > length) {
			return value;
		}


		var tmp = length - this.length;
		if (tmp > chunk.length) {
			chunk += chunk.repeat(tmp / chunk.length);
		}

		return chunk.slice(0, tmp) + value;

	};

}
if (typeof String.prototype.repeat !== 'function') {

	String.prototype.repeat = function(/* count */) {

		if (this === null || this === undefined) {
			throw new TypeError('String.prototype.repeat called on null or undefined');
		}


		var value  = String(this);
		var count  = arguments.length >= 1 ? (arguments[0] | 0) : 0;
		var length = value.length >>> 0;
		var result = '';


		if (count === 0) {
			return result;
		} else if (!isFinite(count)) {
			throw new RangeError('Invalid count value');
		}


		if ((length * count) <= (1 << 28)) {

			for (var c = 0; c < count; c++) {
				result += value;
			}

		}


		return result;

	};

}
if (typeof String.prototype.startsWith !== 'function') {

	String.prototype.startsWith = function(search/*, from */) {

		if (this === null || this === undefined) {
			throw new TypeError('String.prototype.startsWith called on null or undefined');
		}


		var value  = (this).toString();
		var from   = arguments.length >= 2 ? (arguments[1] | 0) : 0;
		var tmp    = String(search);
		var length = tmp.length >>> 0;


		var chunk = value.substr(from, length);
		if (chunk === tmp) {
			return true;
		}


		return false;

	};

}
if (typeof String.prototype.toJSON !== 'function') {

	String.prototype.toJSON = function() {
		return this.valueOf();
	};

}
if (typeof String.prototype.toSource !== 'function') {

	String.prototype.toSource = function() {

		var value = (this).toString();

		value = value.replace('\n', '\\n');
		value = value.replace('\t', '\\t');

		return '(new String("' + value + '"))';

	};

}
if (typeof String.prototype.trim !== 'function') {

	String.prototype.trim = function() {
		return this.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g, "");
	};

}
if (typeof String.prototype.trimEnd !== 'function') {

	String.prototype.trimEnd = function() {
		return this.replace(/[\x09\x0A\x0B\x0C\x0D\x20\xA0\u1680\u180E\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u202F\u205F\u3000\u2028\u2029\uFEFF]*$/, '');
	};

}
if (typeof String.prototype.trimLeft !== 'function') {

	String.prototype.trimLeft = function() {
		return this.replace(/^[\x09\x0A\x0B\x0C\x0D\x20\xA0\u1680\u180E\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u202F\u205F\u3000\u2028\u2029\uFEFF]*/, '');
	};

}
if (typeof String.prototype.trimRight !== 'function') {

	String.prototype.trimRight = function() {
		return this.replace(/[\x09\x0A\x0B\x0C\x0D\x20\xA0\u1680\u180E\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u202F\u205F\u3000\u2028\u2029\uFEFF]*$/, '');
	};

}
if (typeof String.prototype.trimStart !== 'function') {

	String.prototype.trimStart = function() {
		return this.replace(/^[\x09\x0A\x0B\x0C\x0D\x20\xA0\u1680\u180E\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u202F\u205F\u3000\u2028\u2029\uFEFF]*/, '');
	};

}
if (typeof String.toSource !== 'function') {

	String.toSource = function() {
		return 'function String() {\n\t[native code]\n}';
	};

}